@@ -27,7 +27,6 @@
 #include <ncurses.h>
 extern WINDOW *p_win;
 extern WINDOW *bar_win;
-int window_f = 0;
 int color_support = 1;
 #endif
 
@@ -50,20 +49,6 @@ extern void progress_init(struct progress_bar *prog, int start, unsigned long lo
     }
     prog->rate = 0.0;
     prog->pui = PUI;
-
-    /*
-    if (RES){
-        prog->resolution = RES*100;
-    } else {
-        if (stop <= 5000){
-            prog->resolution = 100;
-	} else if ((stop > 5000) && (stop <= 50000)){
-	    prog->resolution = 300;
-        } else {
-            prog->resolution = 1000;
-        }
-    }
-    */
 }
 
 /// open progress interface
@@ -100,15 +85,16 @@ extern void update_pui(struct progress_bar *prog, unsigned long long current, in
 
 static void calculate_speed(struct progress_bar *prog, unsigned long long current, int done, prog_stat_t *prog_stat){
     char *format = "%H:%M:%S";
-    double speedps = 1.0;
-    float speed = 1.0;
+    uint64_t speedps = 1;
+    uint64_t speed = 1;
+    double dspeed = 1.0;
     float percent = 1.0;
     time_t remained;
     time_t elapsed;
     char Rformated[10], Eformated[10];
     char speed_unit[] = "    ";
     struct tm *Rtm, *Etm;
-    uint64_t gbyte=1000000000;
+    uint64_t gbyte=1000000000.0;
     uint64_t mbyte=1000000;
     uint64_t kbyte=1000;
 
@@ -116,31 +102,43 @@ static void calculate_speed(struct progress_bar *prog, unsigned long long curren
     percent  = prog->unit * current;
     if (percent <= 0)
 	percent = 1;
+    else if (percent >= 100)
+	percent = 99.99;
+
 
     elapsed  = (time(0) - prog->initial_time);
     if (elapsed <= 0)
 	elapsed = 1;
 
-    speedps  = (float)prog->block_size * (float)current / (float)(elapsed);
-    speed = (float)(speedps * 60.0);
+    speedps  = prog->block_size * current / elapsed;
+    speed = speedps * 60.0;
 
     if(prog->block_size == 1){ // don't show bitmap rate, bit_size = 1
 	speed = 0;	
     }
 
+    prog_stat->percent   = percent;
+
     if (speed >= gbyte){
-	speed = speed / gbyte;
+	dspeed = (double)speed / (double)gbyte;
 	strncpy(speed_unit, "GB", 3);
+	strncpy(prog_stat->speed_unit, speed_unit, 3);
     }else if (speed >= mbyte){
-	speed = speed / mbyte;
+	dspeed = (double)speed / (double)mbyte;
 	strncpy(speed_unit, "MB", 3);
+	strncpy(prog_stat->speed_unit, speed_unit, 3);
     }else if (speed >= kbyte){
-	speed = speed / kbyte;
+	dspeed = (double)speed / (double)kbyte;
 	strncpy(speed_unit, "KB", 3);
+	strncpy(prog_stat->speed_unit, speed_unit, 3);
     }else{
+	dspeed = speed;
 	strncpy(speed_unit, "byte", 5);
+	strncpy(prog_stat->speed_unit, speed_unit, 5);
     }
 
+    prog_stat->speed     = dspeed;
+
     if (done != 1){
         remained = (time_t)((elapsed/percent*100) - elapsed);
 
@@ -151,7 +149,7 @@ static void calculate_speed(struct progress_bar *prog, unsigned long long curren
         strftime(Eformated, sizeof(Eformated), format, Etm);
 
     } else {
-        percent=100;
+        prog_stat->percent=100;
         remained = (time_t)0;
         Rtm = gmtime(&remained);
         strftime(Rformated, sizeof(Rformated), format, Rtm);
@@ -161,9 +159,6 @@ static void calculate_speed(struct progress_bar *prog, unsigned long long curren
 
     strncpy(prog_stat->Eformated, Eformated, 10);
     strncpy(prog_stat->Rformated, Rformated, 10);
-    prog_stat->percent   = percent;
-    prog_stat->speed     = speed;
-    strncpy(prog_stat->speed_unit, speed_unit, 3);
 }
 
 /// update information at progress bar
@@ -185,28 +180,24 @@ extern void progress_update(struct progress_bar *prog, unsigned long long curren
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 
-        if ((current+1) == prog->stop){
-	    fprintf(stderr, _("\r%80c\rElapsed: %s, Remaining: %s, Completed:%6.2f%%, Seeking...,"), clear_buf, prog_stat.Eformated, prog_stat.Rformated, prog_stat.percent, (float)(prog_stat.speed));
-	} else {
-	    if((int)prog_stat.speed > 0)
-	    fprintf(stderr, _("\r%80c\rElapsed: %s, Remaining: %s, Completed:%6.2f%%, Rate: %6.2f%s/min,"), clear_buf, prog_stat.Eformated, prog_stat.Rformated, prog_stat.percent, (float)(prog_stat.speed), prog_stat.speed_unit);
-	    else
-		fprintf(stderr, _("\r%80c\rElapsed: %s, Remaining: %s, Completed:%6.2f%%,"), clear_buf, prog_stat.Eformated, prog_stat.Rformated, prog_stat.percent);
-	}
+	fprintf(stderr, _("\r%80c\rElapsed: %s, Remaining: %s, Completed:%6.2f%%"), clear_buf, prog_stat.Eformated, prog_stat.Rformated, prog_stat.percent);
+
+	if((int)prog_stat.speed > 0)
+	    fprintf(stderr, _(", %6.2f%s/min,"), prog_stat.speed, prog_stat.speed_unit);
+	//fprintf(stderr, "current block=%lld%s", current, "\x1b[A");
     } else {
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 
+	fprintf(stderr, _("\r%80c\rElapsed: %s, Remaining: %s, Completed: 100.00%%"), clear_buf, prog_stat.Eformated, prog_stat.Rformated);
 	if((int)prog_stat.speed > 0)
-	    fprintf(stderr, _("\r%80c\rElapsed: %s, Remaining: %s, Completed:%6.2f%%, Rate: %6.2f%s/min,"), clear_buf, prog_stat.Eformated, prog_stat.Rformated, prog_stat.percent, (float)(prog_stat.speed), prog_stat.speed_unit);
-	else
-	    fprintf(stderr, _("\r%80c\rElapsed: %s, Remaining: %s, Completed:%6.2f%%,"), clear_buf, prog_stat.Eformated, prog_stat.Rformated, prog_stat.percent);
+	   fprintf(stderr, _(", Rate: %6.2f%s/min,"), prog_stat.speed, prog_stat.speed_unit);
 
         fprintf(stderr, _("\nTotal Time: %s, "), prog_stat.Eformated);
 	if((int)prog_stat.speed > 0)
-	    fprintf(stderr, _("Ave. Rate: %6.1f%s/min, "), (float)(prog_stat.speed), prog_stat.speed_unit);
-        fprintf(stderr, _("%s"), "100.00%% completed!\n");
+	    fprintf(stderr, _("Ave. Rate: %6.1f%s/min, "), prog_stat.speed, prog_stat.speed_unit);
+        fprintf(stderr, _("%s"), "100.00% completed!\n");
     }
 }
 
@@ -215,85 +206,58 @@ extern void Ncurses_progress_update(struct progress_bar *prog, unsigned long lon
 {
 #ifdef HAVE_LIBNCURSESW
 
-    char *clear_buf = NULL;
     char *p_block;
     prog_stat_t prog_stat;
 
     memset(&prog_stat, 0, sizeof(prog_stat_t));
     calculate_speed(prog, current, done, &prog_stat);
 
+    /// set bar color
+    init_pair(4, COLOR_RED, COLOR_RED);
+    init_pair(5, COLOR_WHITE, COLOR_BLUE);
+    init_pair(6, COLOR_WHITE, COLOR_RED);
+    werase(p_win);
+    werase(bar_win);
+
     if (done != 1){
         if (difftime(time(0), prog->resolution_time) < prog->interval_time)
             return;
 	prog->resolution_time = time(0);
 
-        /// set bar color
-        init_pair(4, COLOR_RED, COLOR_RED);
-        init_pair(5, COLOR_WHITE, COLOR_BLUE);
-        init_pair(6, COLOR_WHITE, COLOR_RED);
-        werase(p_win);
-        werase(bar_win);
-
-
         mvwprintw(p_win, 0, 0, _(" "));
         mvwprintw(p_win, 1, 0, _("Elapsed: %s") , prog_stat.Eformated);
         mvwprintw(p_win, 2, 0, _("Remaining: %s"), prog_stat.Rformated);
 	if ((int)prog_stat.speed > 0)
-	    mvwprintw(p_win, 3, 0, _("Rate: %6.2f%s/min"), (float)(prog_stat.speed), prog_stat.speed_unit);
-        //mvwprintw(p_win, 3, 0, _("Completed:%6.2f%%"), percent);
-        p_block = malloc(50);
-        memset(p_block, 0, 50);
+	    mvwprintw(p_win, 3, 0, _("Rate: %6.2f%s/min"), prog_stat.speed, prog_stat.speed_unit);
+        p_block = calloc(sizeof(char), 50);
         memset(p_block, ' ', (size_t)(prog_stat.percent*0.5));
         wattrset(bar_win, COLOR_PAIR(4));
         mvwprintw(bar_win, 0, 0, "%s", p_block);
         wattroff(bar_win, COLOR_PAIR(4));
-        if(prog_stat.percent <= 50){
-            wattrset(p_win, COLOR_PAIR(5));
-            mvwprintw(p_win, 5, 25, "%3.0f%%", prog_stat.percent);
-            wattroff(p_win, COLOR_PAIR(5));
-        }else{
-            wattrset(p_win, COLOR_PAIR(6));
-            mvwprintw(p_win, 5, 25, "%3.0f%%", prog_stat.percent);
-            wattroff(p_win, COLOR_PAIR(6));
-        }
+	free(p_block);
         mvwprintw(p_win, 5, 52, "%6.2f%%", prog_stat.percent);
         wrefresh(p_win);
         wrefresh(bar_win);
-        free(p_block);
     } else {
         mvwprintw(p_win, 1, 0, _("Total Time: %s"), prog_stat.Eformated);
         mvwprintw(p_win, 2, 0, _("Remaining: 0"));
 	if ((int)prog_stat.speed > 0)
-	    mvwprintw(p_win, 3, 0, _("Ave. Rate: %6.2f%s/min"), (float)prog_stat.speed, prog_stat.speed_unit);
+	    mvwprintw(p_win, 3, 0, _("Ave. Rate: %6.2f%s/min"), prog_stat.speed, prog_stat.speed_unit);
+
         wattrset(bar_win, COLOR_PAIR(4));
-        mvwprintw(bar_win, 0, 0, "%50s", " ");
+	mvwprintw(bar_win, 0, 0, "%50s", " ");
         wattroff(bar_win, COLOR_PAIR(4));
-        wattrset(p_win, COLOR_PAIR(6));
-        mvwprintw(p_win, 5, 22, "%6.2f%%", prog_stat.percent);
-        wattroff(p_win, COLOR_PAIR(6));
-        mvwprintw(p_win, 5, 52, "%6.2f%%", prog_stat.percent);
+
+        mvwprintw(p_win, 5, 52, "100.00%%");
         wrefresh(p_win);
         wrefresh(bar_win);
         refresh();
-        sleep(1);
-    }
-
-    if(done == 1){
-        window_f = close_p_ncurses();
+	sleep(1);
     }
 
 #endif
 }
 
-static int open_p_ncurses(){
-
-    return 1;
-}
-
-static int close_p_ncurses(){
-
-    return 1;
-}
 /// update information as dialog format, refernece source code of dialog
 /// # mkfifo pipe
 /// # (./clone.extfs -d -c -X -s /dev/loop0 2>pipe | cat - > test.img) | ./gauge < pipe
